unit Angledit;

interface

uses WinTypes, WinProcs, Classes, Graphics, Forms, Controls, Pies,
  Buttons, DsgnIntf, StdCtrls;

type
  TAngleEditorDlg = class(TForm)
    EAngleBar: TScrollBar;
    EAngleLabel: TLabel;
    OKButton: TBitBtn;
    CancelButton: TBitBtn;
    SAngleLabel: TLabel;
    SAngleBar: TScrollBar;
    procedure EAngleBarScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure SAngleBarScroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure CancelClick(Sender: TObject);
  private
    FOrigStart, FOrigEnd: Integer;
    FAngles: TAngles;
    procedure SetStartAngle(Value: Integer);
    procedure SetEndAngle(Value: Integer);
    procedure SetAngles(Value: TAngles);
  public
    property EditorAngles: TAngles read FAngles write SetAngles;
  end;

  TAnglesProperty = class(TClassProperty)
  public
    procedure Edit; override;
    function GetAttributes: TPropertyAttributes; override;
  end;

  { Component editor - brings up angle editor when double clicking on
    Angles property }
  TPieEditor = class(TDefaultEditor)
  public
    procedure EditProperty(PropertyEditor: TPropertyEditor;
      var Continue, FreeEditor: Boolean); virtual;
  end;

implementation

uses SysUtils;

{$R *.DFM}

{ TAngleEditorDlg }

procedure TAngleEditorDlg.SAngleBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  SetStartAngle(ScrollPos);
end;

procedure TAngleEditorDlg.EAngleBarScroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  SetEndAngle(ScrollPos);
end;

procedure TAngleEditorDlg.SetStartAngle(Value: Integer);
begin
  SAngleBar.Position := Value;
  SAngleLabel.Caption := 'StartAngle = '+ IntToStr(Value);
  FAngles.StartAngle := Value;
end;

procedure TAngleEditorDlg.SetEndAngle(Value: Integer);
begin
  EAngleBar.Position := Value;
  EAngleLabel.Caption := 'EndAngle = '+ IntToStr(Value);
  FAngles.EndAngle := Value;
end;

procedure TAngleEditorDlg.SetAngles(Value: TAngles);
begin
  FAngles := Value;
  FOrigStart := Value.StartAngle;
  FOrigEnd := Value.EndAngle;
  SetStartAngle(Value.StartAngle);
  SetEndAngle(Value.EndAngle);
end;

procedure TAngleEditorDlg.CancelClick(Sender: TObject);
begin
  SetStartAngle(FOrigStart);
  SetEndAngle(FOrigEnd);
end;

{ TAnglesProperty }

procedure TAnglesProperty.Edit;
var
  Angles: TAngles;
  AngleEditor: TAngleEditorDlg;
begin
  Angles := TAngles(GetOrdValue);
  AngleEditor := TAngleEditorDlg.Create(Application);
  try
    AngleEditor.EditorAngles := Angles;
    AngleEditor.ShowModal;
  finally
    AngleEditor.Free;
  end;
end;

function TAnglesProperty.GetAttributes: TPropertyAttributes;
begin
  Result := [paDialog, paSubProperties];
end;

{TPieEditor}

procedure TPieEditor.EditProperty(PropertyEditor: TPropertyEditor;
  var Continue, FreeEditor: Boolean);
var
  PropName: string;
begin
  PropName := PropertyEditor.GetName;
  if (CompareText(PropName, 'ANGLES') = 0) then
  begin
    PropertyEditor.Edit;
    Continue := False;
  end;
end;

end.
